#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2009-2019 German Aerospace Center (DLR) and others.
# This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v2.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v20.html
# SPDX-License-Identifier: EPL-2.0

# @file    runner.py
# @author  Lena Kalleske
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @author  Jakob Erdmann
# @date    2009-03-26
# @version $Id$

from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import optparse
import random

# we need to import python modules from the $SUMO_HOME/tools directory
if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")

from sumolib import checkBinary  # noqa
import traci  # noqa


def generate_routefile():
    random.seed(18)  # make tests reproducible
    N = 5000  # number of time steps
    # demand per second from different directions
    West = 0.0
    East = 0.0
    North = 0.0
    South = 0.0
    
    with open("data/cross.rou.xml", "w") as routes:
        print("""<routes>
        <vType id="typeACC" accel="2.6" decel="4.5" emergencyDecel="9" length="5" minGap="5" maxSpeed="8.33333" guiShape="passenger" carFollowModel="ACC" tau="1" sigma="0.5" eClass="HBEFA3/PC_G_EU4" probability="0.5" color="1, 1, 0" /> 
        
        <vType id="typeHD" accel="2.6" decel="4.5" emergencyDecel="9" length="5" minGap="5" maxSpeed="8.33333" guiShape="passenger" delta="4" tau="1" sigma="0.5" eClass="HBEFA3/PC_G_EU4" probability="0.5" color="1, 0, 0" />
        <vTypeDistribution id="typedist1" vTypes="typeHD typeACC"/> 
        
        <routeDistribution id="fromWest1">
        <route id="W1N1" edges="gneE2 gneE3" probability="0.2"/>
        <route id="W1N2" edges="gneE2 0-1 gneE7" probability="0.2"/>
        <route id="W1S1" edges="gneE2 0-2 gneE15" probability="0.2"/>
        <route id="W1E1" edges="gneE2 0-1 gneE5" probability="0.2"/>
        <route id="W1W2" edges="gneE2 0-2 gneE17" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromWest2">
        <route id="W2N1" edges="gneE18 2-0 gneE3" probability="0.2"/>
        <route id="W2S1" edges="gneE18 gneE15" probability="0.2"/>
        <route id="W2S2" edges="gneE18 2-3 gneE13" probability="0.2"/>
        <route id="W2E2" edges="gneE18 2-3 gneE10" probability="0.2"/>
        <route id="W2W1" edges="gneE18 2-0 gneE0" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromEast1">
        <route id="E1N2" edges="gneE9 gneE7" probability="0.2"/>
        <route id="E1N1" edges="gneE9 1-0 gneE3" probability="0.2"/>
        <route id="E1W1" edges="gneE9 1-0 gneE0" probability="0.2"/>
        <route id="E1S2" edges="gneE9 1-3 gneE13" probability="0.2"/>
        <route id="E1E2" edges="gneE9 1-3 gneE10" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromEast2">
        <route id="E2S2" edges="gneE11 gneE13" probability="0.2"/>
        <route id="E2S1" edges="gneE11 3-2 gneE15" probability="0.2"/>
        <route id="E2W2" edges="gneE11 3-2 gneE17" probability="0.2"/>
        <route id="E2N2" edges="gneE11 3-1 gneE7" probability="0.2"/>
        <route id="E2E1" edges="gneE11 3-1 gneE5" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromNorth1">
        <route id="N1W1" edges="gneE4 gneE0" probability="0.2"/>
        <route id="N1N2" edges="gneE4 0-1 gneE7" probability="0.2"/>
        <route id="N1S1" edges="gneE4 0-2 gneE15" probability="0.2"/>
        <route id="N1E1" edges="gneE4 0-1 gneE5" probability="0.2"/>
        <route id="N1W2" edges="gneE4 0-2 gneE17" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromNorth2">
        <route id="N2W1" edges="gneE8 1-0 gneE0" probability="0.2"/>
        <route id="N2N1" edges="gneE8 1-0 gneE3" probability="0.2"/>
        <route id="N2S2" edges="gneE8 1-3 gneE13" probability="0.2"/>
        <route id="N2E2" edges="gneE8 1-3 gneE10" probability="0.2"/>
        <route id="N2E1" edges="gneE8 gneE5" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromSouth1">
        <route id="S1N1" edges="gneE16 2-0 gneE3" probability="0.2"/>
        <route id="S1W1" edges="gneE16 2-0 gneE0" probability="0.2"/>
        <route id="S1W2" edges="gneE16 gneE17" probability="0.2"/>
        <route id="S1S2" edges="gneE16 2-3 gneE13" probability="0.2"/>
        <route id="S1E2" edges="gneE16 2-3 gneE10" probability="0.2"/>
        </routeDistribution>
        
        <routeDistribution id="fromSouth2">
        <route id="S2S1" edges="gneE14 3-2 gneE15" probability="0.2"/>
        <route id="S2W2" edges="gneE14 3-2 gneE17" probability="0.2"/>
        <route id="S2E2" edges="gneE14 gneE10" probability="0.2"/>
        <route id="S2E1" edges="gneE14 3-1 gneE5" probability="0.2"/>
        <route id="S2N2" edges="gneE14 3-1 gneE7" probability="0.2"/>
        </routeDistribution>
        
        """, file=routes)
        vehNr = 0
        for i in range(N):
            if random.uniform(0, 1) < West:
                print('    <vehicle id="right_%i" type="typedist1" route="fromWest1" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < East:
                print('    <vehicle id="right_%i" type="typedist1" route="fromEast1" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < North:
                print('    <vehicle id="right_%i" type="typedist1" route="fromNorth1" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < South:
                print('    <vehicle id="right_%i" type="typedist1" route="fromSouth1" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            
            if random.uniform(0, 1) < West:
                print('    <vehicle id="right_%i" type="typedist1" route="fromWest2" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < East:
                print('    <vehicle id="right_%i" type="typedist1" route="fromEast2" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < North:
                print('    <vehicle id="right_%i" type="typedist1" route="fromNorth2" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1
            if random.uniform(0, 1) < South:
                print('    <vehicle id="right_%i" type="typedist1" route="fromSouth2" depart="%i" />' % (
                    vehNr, i), file=routes)
                vehNr += 1            
        print("</routes>", file=routes)        

# The program looks like this
#    <tlLogic id="0" type="static" programID="0" offset="0">
# the locations of the tls are      NESW
#        <phase duration="31" state="GrGr"/>
#        <phase duration="6"  state="yryr"/>
#        <phase duration="31" state="rGrG"/>
#        <phase duration="6"  state="ryry"/>
#    </tlLogic>

#<vType id="AV" accel="2.6" decel="4.5" emergencyDecel="9" length="5" minGap="5" maxSpeed="8.333" vClass="passenger" probability="0.0" carFollowModel="ACC"  sigma="0" tau="1" color="1, 1, 0" eClass="HBEFA3/PC_G_EU4"  lcLaneDiscipline="1" latAlignment="center"/>
		
#<route id="N1N2" edges="gneE4 0-1 gneE7" />
#		<route id="N1E1" edges="gneE4 0-1 gneE5"  />
#        <route id="N1S1" edges="gneE4 0-2 gneE15"  />
#        <route id="N1W2" edges="gneE4 0-2 gneE17"  />
#		<route id="N1E2" edges="gneE4 0-1 1-3 gneE10"  />
#        <route id="N1S2" edges="gneE4 0-1 1-3 gneE13"  />

def run():
    """execute the TraCI control loop"""
    step = 0
    # we start with phase 2 where EW has green
    traci.trafficlight.setProgram("0", "0")
    traci.trafficlight.setProgram("1", "0")
    traci.trafficlight.setProgram("2", "0")
    traci.trafficlight.setProgram("3", "0")
    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()
        
        step += 1
    traci.close()
    sys.stdout.flush()


def get_options():
    optParser = optparse.OptionParser()
    optParser.add_option("--nogui", action="store_true",
                         default=False, help="run the commandline version of sumo")
    options, args = optParser.parse_args()
    return options


# this is the main entry point of this script
if __name__ == "__main__":
    options = get_options()

    # this script has been called from the command line. It will start sumo as a
    # server, then connect and run
    if options.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    # first, generate the route file for this simulation
    generate_routefile()

    # this is the normal way of using traci. sumo is started as a
    # subprocess and then the python script connects and runs
    traci.start([sumoBinary, "-c", "data/cross.sumocfg",
                             "--emission-output", "Scenario-6.xml"])
    run()
